@@ -72,6 +72,8 @@ class Bytecode2SpirVGenerator(OpCodeDefinitions, BaseSpirVGenerator):
         self._texture = {}
         self._slotmap = {}  # (namespaceidentifier, slot) -> name
 
+        self._decorated_array_types = set()
+
         # We keep track of sampler for each combination of texture and sampler
         self._texture_samplers = {}
 
@@ -348,7 +350,7 @@ class Bytecode2SpirVGenerator(OpCodeDefinitions, BaseSpirVGenerator):
         elif funcname == "matrix_inverse":
             nargs = 1
             result_type = ty = args[0].type
-            if issubclass(ty, _types.Matrix) and ty._rows == ty.cols:
+            if issubclass(ty, _types.Matrix) and ty.rows == ty.cols:
                 nr = 34
             else:
                 raise ShaderError("matrix_inverse() expects square matrix.")
@@ -586,6 +588,20 @@ class Bytecode2SpirVGenerator(OpCodeDefinitions, BaseSpirVGenerator):
         elif kind == "buffer":
             # todo: according to docs, in SpirV 1.4+, BufferBlock is deprecated
             # and one should use Block with StorageBuffer. But this crashes.
+            # Generate an ArrayStride on the storage buffer array
+            if issubclass(var_type, _types.Struct) and len(var_type.keys) == 1:
+                array_type = var_type.get_subtype(0)
+                if issubclass(array_type, _types.Array):
+                    stride = ctypes.sizeof(array_type.subtype._as_ctype())
+                    if stride > 0 and array_type not in self._decorated_array_types:
+                        self._decorated_array_types.add(array_type)
+                        self.gen_instruction(
+                            "annotations",
+                            cc.OpDecorate,
+                            self.obtain_type_id(array_type),
+                            cc.Decoration_ArrayStride,
+                            stride,
+                        )
             self.gen_instruction(
                 "annotations", cc.OpDecorate, var_id, cc.Decoration_BufferBlock
             )
@@ -822,14 +838,15 @@ class Bytecode2SpirVGenerator(OpCodeDefinitions, BaseSpirVGenerator):
             self._stack.append(ac)
         elif issubclass(ob.type, _types.Vector):
             indices = []
+            # Wr support xyzw, rgba, stpq
             for c in name:
-                if c in "xr":
+                if c in "xrs":
                     indices.append(0)
-                elif c in "yg":
+                elif c in "ygt":
                     indices.append(1)
-                elif c in "zb":
+                elif c in "zbp":
                     indices.append(2)
-                elif c in "wa":
+                elif c in "waq":
                     indices.append(3)
                 else:
                     raise ShaderError(f"Invalid vector attribute {name}")
@@ -952,7 +969,14 @@ class Bytecode2SpirVGenerator(OpCodeDefinitions, BaseSpirVGenerator):
         elif type1 is type2 and issubclass(type1, scalar_or_vector):
             # Types are equal and scalar or vector. Covers a lot of cases.
             result_id, type_id = self.obtain_value(type1)
-            if issubclass(reftype1, _types.Float):
+            if (
+                issubclass(type1, _types.Vector)
+                and issubclass(reftype1, _types.Float)
+                and op == "mmul"
+            ):
+                opcode = cc.OpDot  # special case
+                result_id, type_id = self.obtain_value(type1.subtype)
+            elif issubclass(reftype1, _types.Float):
                 opcode = FOPS[op]
             elif issubclass(reftype1, _types.Int):
                 opcode = IOPS[op]
